<?php

abstract class Upload
{
    public static $dir_image_cache = 'media/cache/images';

    public static function getGreagwarImage($file_path)
    {
        require_once PATH . '/lib/GreagwarImage/GreagwarImage.php';
        $greagwar_image = new GreagwarImage($file_path);
        $greagwar_image->setCacheDir(Config::get('PATH_WEB_ROOT') . DIRECTORY_SEPARATOR . self::$dir_image_cache);
        return $greagwar_image;
    }

    /**
     * @param $class string|Image
     * @param $file array (ex. ['tmp_name' => '...', 'name' => '...', 'type' => '...'])
     * @param $force_create_variants bool
     * @param int $quality
     * @return Image
     * @throws ErrorException
     * TODO: Рефакторинг: убрать параметр $force_create_variants
     */
    public static function image($class, $file, $force_create_variants = true, $quality = 87)
    {
        /**
         * @var $image Image
         */
        if (is_null($class) || $class == '') {
            throw new ErrorException('Class not defined.');
        }
        if (!is_object($class) && !class_exists($class)) {
            throw new ErrorException('Class "' . $class . '" not exists.');
        }
        Image::checkSubClass($class);
        $image = is_object($class) ? $class : new $class;
        $image->original_filename = $file['name'];

        $greagwar_image = self::getGreagwarImage($file['tmp_name']);
        self::saveImage($image, $greagwar_image, $class::getMaxWidth(), $class::getMaxHeight(), $quality);
        if ($force_create_variants) {
            $sizes = $image->getSizes();
            foreach ($sizes as $size) {
                $size_parts = explode('x', $size);
                self::saveImage($image->getImageVariant($size), $greagwar_image, $size_parts[0] ? : null, $size_parts[1] ? : null, $quality);
            }
        }
        // В случае, если был передан объект, возвращение результата не имеет значения
        return $image;
    }

    /**
     * @param $image ImageVariant|Image
     * @param $greagwar_image GreagwarImage
     * @param null $width
     * @param null $height
     * @param int $quality
     * TODO: Сделать возможность передавать параметры для метода _resize()
     */
    public static function saveImage($image, $greagwar_image, $width = null, $height = null, $quality = 87)
    {
        if (!ImageVariant::getIsClass($image) && is_null($width) && is_null($height)) {
            if (Image::getIsSubClass($image)) {
                $width = $image::getMaxWidth();
                $height = $image::getMaxHeight();
            }
        }
        $greagwar_image = clone($greagwar_image);
        if ($width || $height) {
            $greagwar_image->resize($width, $height, 0xffffff, $force = true, $rescale = true, $crop = false);
        }
        $image->type = $greagwar_image->guessType();
        $file_path = $greagwar_image->cacheFile($image->type, $quality, true);
        $image->size = filesize($file_path);
        $path_parts = pathinfo($file_path);
        $image->path = preg_replace('#^' . Config::get('PATH_WEB_ROOT') . '/#', '', $path_parts['dirname']);
        $image->filename = $path_parts['basename'];
        $image->width = $greagwar_image->width();
        $image->height = $greagwar_image->height();
    }
}